/*============================================================================
  File:     Covering and INCLUDE.sql

  Summary:  Various examples of nonclustered indexes that cover queries as
			well as better uses of INCLUDE.
  
  Date:     April 2010

  SQL Server Version: SQL Server 2005/2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

----------------------------------------------------------------------------
 -- Resource: SQL Server 2008 Internals (Chapter 6, Index Internals)
 --           http://www.microsoft.com/learning/en/us/books/12967.aspx

 --  Summary: This script uses a couple "sample" databases.
 --				IndexInternals sample database can be downloaded from here:
 --				  http://www.sqlskills.com/blogs/Kimberly/post/Companion-content-for-Chapter-6-(Index-Internals)-of-SQL-Server-2008-Internals.aspx				
 --				Credit sample database can be downloaded from here:
 --				  http://www.sqlskills.com/pastConferences.asp
----------------------------------------------------------------------------

USE IndexInternals
go

SET STATISTICS IO ON
go

SELECT e.EmployeeID, e.SSN 
FROM dbo.Employee AS e 
WHERE e.SSN BETWEEN '497-07-9812' 
	AND '500-00-1194'
go

SELECT e.EmployeeID, e.SSN 
FROM dbo.Employee AS e 
WHERE e.EmployeeID < 10000
go

-- Compare against the seekable clustered index

SELECT e.EmployeeID, e.SSN 
FROM dbo.Employee AS e WITH (INDEX (1))
WHERE e.EmployeeID < 10000
go

SELECT e.EmployeeID, e.SSN 
FROM dbo.Employee AS e 
WHERE e.EmployeeID < 10000
go

-- Covering examples:

USE CREDIT
go

CREATE INDEX NCIndexLNOnly
ON dbo.member(lastname)

CREATE INDEX NCIndexCoversAll4Cols 
ON dbo.member 
(lastname, firstname, middleinitial, phone_no)

CREATE INDEX NCIndexLNinKeyInclude3OtherCols 
ON dbo.member(lastname)
INCLUDE (firstname, middleinitial, phone_no)

CREATE INDEX NCIndexCoveringLnFnMiIncludePhone
ON dbo.member(lastname, firstname, middleinitial)
INCLUDE (phone_no)
go

-- Force the table scan
SELECT m.LastName, m.FirstName, m.Phone_No
FROM dbo.Member AS m WITH (INDEX (0))
WHERE m.LastName LIKE '[S-Z]%'
go

-- Force the lookups
SELECT m.LastName, m.FirstName, m.Phone_No
FROM dbo.Member AS m WITH (INDEX (NCIndexLNOnly))
WHERE m.LastName LIKE '[S-Z]%'
go

-- Leveraging covering
SELECT m.LastName, m.FirstName, m.Phone_No
FROM dbo.Member AS m WITH (INDEX (NCIndexCoversAll4Cols))
WHERE m.LastName LIKE '[S-Z]%'
go

SELECT m.LastName, m.FirstName, m.Phone_No
FROM dbo.Member AS m WITH (INDEX (NCIndexLNinKeyInclude3OtherCols ))
WHERE m.LastName LIKE '[S-Z]%'
go

SELECT m.LastName, m.FirstName, m.Phone_No
FROM dbo.Member AS m WITH (INDEX (NCIndexCoveringLnFnMiIncludePhone))
WHERE m.LastName LIKE '[S-Z]%'
go

-- Reviewing the sizes:
SELECT * FROM sys.dm_db_index_physical_stats
	(db_id(), object_id('Member'), NULL, NULL, 'detailed')

-- What about an ORDER BY?
SELECT m.LastName, m.FirstName, m.Phone_No
FROM dbo.Member AS m WITH (INDEX (NCIndexCoversAll4Cols))
WHERE m.LastName LIKE '[S-Z]%'
ORDER BY m.LastName, m.FirstName, m.middleinitial
go

SELECT m.LastName, m.FirstName, m.Phone_No
FROM dbo.Member AS m WITH (INDEX (NCIndexLNinKeyInclude3OtherCols ))
WHERE m.LastName LIKE '[S-Z]%'
ORDER BY m.LastName, m.FirstName, m.middleinitial
go

SELECT m.LastName, m.FirstName, m.Phone_No
FROM dbo.Member AS m WITH (INDEX (NCIndexCoveringLnFnMiIncludePhone))
WHERE m.LastName LIKE '[S-Z]%'
ORDER BY m.LastName, m.FirstName, m.middleinitial
go

-- What about other queries that might want to seek?
SELECT * 
FROM dbo.member AS m
WHERE m.LastName = 'PAYNTER' 
	AND	m.firstname LIKE 'K%'
	
-- As long as you use LEFT-BASED subsets of the key, you can
-- use it for seeking!

-- If you want to clean up the indexes:
--DROP INDEX Member.MemberLastName
--DROP INDEX Member.NCLastNameCombo
--DROP INDEX Member.NCLastNameCombo2